@@ -6,7 +6,7 @@
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
@@ -50,9 +50,21 @@
  *
  */
 
-/********
 
-Mod_Auth_External version 2.1.18
+/********
+ *
+ * Mod_Auth_External version 2.2.11
+ *
+ * Original External authentication module by Nathan Neulinger <nneul@umr.edu>
+ *
+ * Versions through 2.0.1 by Nathan Neulinger <nneul@umr.edu>
+ *                       and Tyler Allison <allison@nas.nasa.gov>
+ *
+ * Versions after 2.1.0 by Jan Wolter <jan@unixpapa.com>
+ *
+ * Port to Apache 2.0 by Dave Woolaway <dave@watersheep.org>
+ *                   and Sven Koch <haegar@sdinet.de>
+
 
 Add to server configuration file (normally httpd.conf):
 -------------------------------------------------------
@@ -76,7 +88,7 @@ Add to server configuration file (normally httpd.conf):
 	checkpassword   write null-terminated strings to filedescriptor 3.
 	function        call an internal function instead of external program.
 
- For "environment", "pipe" and "checkpassword", the option specifies a 
+ For "environment", "pipe" and "checkpassword", the option specifies a
  program to run.  We do not start a shell to run the command, so full path
  must be given and no special shell characters (wildcards, I/O redirection,
  etc.) can be used.  If you need these, write an sh-script wrapper for
@@ -200,26 +212,29 @@ Comments/questions/etc. to janc@cyberspace.org
   /* Uncomment if you want to use your own Hardcode (default off) */
   /*             MUST HAVE _HARDCODE_ defined above!                */
   /* #include "your_function_here.c" */
-#endif 
+#endif
 
 
-/*
- * Original External authentication module by nneul@umr.edu
- * Versions through 2.0.1 by <nneul@umr.edu & allison@nas.nasa.gov>
- * Versions after 2.1.0 by janc@cyberspace.org
- */
+#include "apr_lib.h"
 
+#define APR_WANT_STRFUNC
+#include "apr_want.h"
+#include "apr_strings.h"
 
 #include "httpd.h"
 #include "http_config.h"
 #include "http_core.h"
 #include "http_log.h"
 #include "http_protocol.h"
+#include "http_request.h"   /* for ap_hook_(check_user_id | auth_checker)*/
 #include <sys/wait.h>
 #include <signal.h>
+#if APR_HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
-#if !defined(APACHE_RELEASE) || (APACHE_RELEASE < 1030100)
-#error This module requires Apache 1.3.1 or later.
+#ifndef STANDARD20_MODULE_STUFF
+#error This module requires Apache 2.0.0 or later.
 #endif
 
 /* Names of environment variables used to pass data to authenticator */
@@ -229,7 +244,7 @@ Comments/questions/etc. to janc@cyberspace.org
 #define ENV_URI		"URI"
 #define ENV_IP		"IP"
 #define ENV_HOST	"HOST"		/* Remote Host */
-#define ENV_HTTP_HOST	"HTTP_HOST"	/* Local Host */
+#define ENV_HTTP_HOST   "HTTP_HOST"	/* Local Host */
 /* Define this if you want cookies passed to the script */
 #define ENV_COOKIE	"COOKIE"
 
@@ -239,8 +254,7 @@ Comments/questions/etc. to janc@cyberspace.org
 /*
  * Structure for the module itself
  */
-
-module external_auth_module;
+module AP_MODULE_DECLARE_DATA auth_external_module;
 
 /*
  *  Data types for per-dir and server configuration
@@ -253,13 +267,13 @@ typedef struct
     int  external_groupsatonce;
 } extauth_dir_config_rec;
 
-typedef struct 
+typedef struct
 {
-    table *auth_extpath;
-    table *auth_extmethod;
+    apr_table_t *auth_extpath;
+    apr_table_t *auth_extmethod;
 
-    table *group_extpath;
-    table *group_extmethod;
+    apr_table_t *group_extpath;
+    apr_table_t *group_extmethod;
 } extauth_server_config_rec;
 
 
@@ -267,9 +281,9 @@ typedef struct
  * Creators for per-dir and server configurations
  */
 
-static void *create_extauth_dir_config(pool *p, char *d)
+static void *create_extauth_dir_config(apr_pool_t *p, char *d)
 {
-    extauth_dir_config_rec *r= ap_pcalloc(p, sizeof(extauth_dir_config_rec));
+    extauth_dir_config_rec *r= apr_pcalloc(p, sizeof(extauth_dir_config_rec));
 
     r->external_authoritative= 1;	/* strong by default */
     r->external_groupsatonce= 1;	/* default to on */
@@ -277,17 +291,17 @@ static void *create_extauth_dir_config(pool *p, char *d)
 }
 
 
-static void *create_extauth_server_config( pool *p, server_rec *s)
+static void *create_extauth_server_config( apr_pool_t *p, server_rec *s)
 {
     extauth_server_config_rec *scr;
 
-    scr= (extauth_server_config_rec *) ap_palloc(p,
+    scr= (extauth_server_config_rec *) apr_palloc(p,
     	sizeof(extauth_server_config_rec) );
 
-    scr->auth_extmethod= ap_make_table(p, 4);
-    scr->auth_extpath= ap_make_table(p, 4);
-    scr->group_extmethod= ap_make_table(p, 4);
-    scr->group_extpath= ap_make_table(p, 4);
+    scr->auth_extmethod=  apr_table_make(p, 4);
+    scr->auth_extpath=    apr_table_make(p, 4);
+    scr->group_extmethod= apr_table_make(p, 4);
+    scr->group_extpath=   apr_table_make(p, 4);
 
     return (void *)scr;
 }
@@ -298,17 +312,17 @@ static void *create_extauth_server_config( pool *p, server_rec *s)
  * type to the server configuration
  */
 
-static const char *add_extauth(cmd_parms *cmd, void *dummy, char *keyword,
-				char *path)
+static const char *add_extauth(cmd_parms *cmd, void *dummy, const char *keyword,
+				const char *path)
 {
     extauth_server_config_rec *sc_rec;
 
     sc_rec= ap_get_module_config( cmd->server->module_config,
-	&external_auth_module);
+	&auth_external_module);
 
-    ap_table_set( sc_rec->auth_extpath, keyword, path );
+    apr_table_set( sc_rec->auth_extpath, keyword, path );
 
-    ap_table_set( sc_rec->auth_extmethod, keyword, "environment" );
+    apr_table_set( sc_rec->auth_extmethod, keyword, "environment" );
 
     return NULL;
 }
@@ -318,17 +332,17 @@ static const char *add_extauth(cmd_parms *cmd, void *dummy, char *keyword,
  * type to the server configuration
  */
 
-static const char *add_extgroup(cmd_parms *cmd, void *dummy, char *keyword,
-				char *path)
+static const char *add_extgroup(cmd_parms *cmd, void *dummy,
+			 	const char *keyword, const char *path)
 {
     extauth_server_config_rec *sc_rec;
 
     sc_rec= ap_get_module_config( cmd->server->module_config,
-	&external_auth_module);
+	&auth_external_module);
 
-    ap_table_set( sc_rec->group_extpath, keyword, path );
+    apr_table_set( sc_rec->group_extpath, keyword, path );
 
-    ap_table_set( sc_rec->group_extmethod, keyword, "environment" );
+    apr_table_set( sc_rec->group_extmethod, keyword, "environment" );
 
     return NULL;
 }
@@ -339,14 +353,14 @@ static const char *add_extgroup(cmd_parms *cmd, void *dummy, char *keyword,
  */
 
 static const char *set_extauth_method(cmd_parms *cmd, void *dummy,
-					char *keyword, char *method)
+					const char *keyword, const char *method)
 {
 	extauth_server_config_rec *sc_rec;
 
 	sc_rec= ap_get_module_config( cmd->server->module_config,
-	    &external_auth_module);
+	    &auth_external_module);
 
-	ap_table_set( sc_rec->auth_extmethod, keyword, method );
+	apr_table_set( sc_rec->auth_extmethod, keyword, method );
 	
 	return NULL;
 }
@@ -357,14 +371,14 @@ static const char *set_extauth_method(cmd_parms *cmd, void *dummy,
  */
 
 static const char *set_extgroup_method(cmd_parms *cmd, void *dummy,
-					char *keyword, char *method)
+					const char *keyword, const char *method)
 {
 	extauth_server_config_rec *sc_rec;
 
 	sc_rec= ap_get_module_config( cmd->server->module_config,
-	    &external_auth_module);
+	    &auth_external_module);
 
-	ap_table_set( sc_rec->group_extmethod, keyword, method );
+	apr_table_set( sc_rec->group_extmethod, keyword, method );
 	
 	return NULL;
 }
@@ -374,39 +388,46 @@ static const char *set_extgroup_method(cmd_parms *cmd, void *dummy,
  * Commands that this module can handle
  */
 
-command_rec extauth_cmds[]= {
+static const command_rec extauth_cmds[] =
+{
+	AP_INIT_TAKE1("AuthExternal", ap_set_string_slot,
+	    (void *)APR_OFFSETOF(extauth_dir_config_rec,auth_extname),
+	    OR_AUTHCFG,
+	    "a keyword indicating which authenticator to use"),
 
-	{ "AuthExternal", ap_set_string_slot, 
-	(void*)XtOffsetOf(extauth_dir_config_rec,auth_extname),
-	OR_AUTHCFG, TAKE1, "a keyword indicating which authenticator to use" },
+	AP_INIT_TAKE2("AddExternalAuth", add_extauth, NULL,
+	    RSRC_CONF,
+	    "a keyword followed by a path to the authenticator program"),
+	    
+	AP_INIT_TAKE2("SetExternalAuthMethod", set_extauth_method, NULL,
+	    RSRC_CONF,
+	    "a keyword followed by the method by which the data is passed"),
 
-	{ "AddExternalAuth", add_extauth, NULL, RSRC_CONF, TAKE2,
-	"a keyword followed by a path to the authenticator program" },
+	AP_INIT_TAKE1("GroupExternal", ap_set_string_slot,
+	    (void *)APR_OFFSETOF(extauth_dir_config_rec,group_extname),
+	    OR_AUTHCFG,
+	    "a keyword indicating which group checker to use"),
+		
+	AP_INIT_TAKE2("AddExternalGroup", add_extgroup, NULL,
+	    RSRC_CONF,
+	    "a keyword followed by a path to the group check program"),
 
-        { "SetExternalAuthMethod", set_extauth_method, NULL, RSRC_CONF, TAKE2,
-        "a keyword followed by the method by which the data is passed" },
+	AP_INIT_TAKE2("SetExternalGroupMethod", set_extgroup_method, NULL,
+	    RSRC_CONF, 
+	    "a keyword followed by the method by which the data is passed"),
 
-	{ "GroupExternal", ap_set_string_slot, 
-	(void*)XtOffsetOf(extauth_dir_config_rec,group_extname),
-	OR_AUTHCFG, TAKE1, "a keyword indicating which group checker to use" },
+	AP_INIT_FLAG("AuthExternalAuthoritative", ap_set_flag_slot,
+	    (void *)APR_OFFSETOF(extauth_dir_config_rec,
+		external_authoritative),
+	    OR_AUTHCFG,
+	    "Set to 'off' to allow access control to be passed along to lower "
+	    "modules if the UserID is not known to this module" ),
 
-	{ "AddExternalGroup", add_extgroup, NULL, RSRC_CONF, TAKE2,
-	"a keyword followed by a path to the group check program" },
-
-        { "SetExternalGroupMethod", set_extgroup_method, NULL, RSRC_CONF, TAKE2,
-        "a keyword followed by the method by which the data is passed" },
-
-        { "AuthExternalAuthoritative", ap_set_flag_slot,
-          (void*)XtOffsetOf(extauth_dir_config_rec, external_authoritative),
-          OR_AUTHCFG, FLAG,
-          "Set to 'off' to allow access control to be passed along to lower "
-          "modules if the UserID is not known to this module" },
-
-        { "AuthExternalGroupsAtOnce", ap_set_flag_slot,
-          (void*)XtOffsetOf(extauth_dir_config_rec, external_groupsatonce),
-          OR_AUTHCFG, FLAG,
-          "Set to 'off' if group authenticator cannot handle multiple group "
-	  "names in one invocation" },
+	AP_INIT_FLAG("AuthExternalGroupsAtOnce", ap_set_flag_slot,
+	    (void *)APR_OFFSETOF(extauth_dir_config_rec, external_groupsatonce),
+	    OR_AUTHCFG,
+	    "Set to 'off' if group authenticator cannot handle multiple group "
+	    "names in one invocation" ),
 
 	{ NULL }
 };
@@ -417,12 +438,12 @@ command_rec extauth_cmds[]= {
 
 static char *lookup_header(const request_rec *r, const char *name)
 {
-    array_header *hdrs_arr;
-    table_entry *hdrs;
+    const apr_array_header_t *hdrs_arr;
+    const apr_table_entry_t *hdrs;
     int i;
 
-    hdrs_arr= ap_table_elts(r->headers_in);
-    hdrs= (table_entry *)hdrs_arr->elts;
+    hdrs_arr= apr_table_elts(r->headers_in);
+    hdrs= (const apr_table_entry_t *)hdrs_arr->elts;
     for (i= 0; i < hdrs_arr->nelts; ++i)
     {
 	if (hdrs[i].key != NULL && strcasecmp(hdrs[i].key, name) == 0)
@@ -436,7 +457,7 @@ static char *lookup_header(const request_rec *r, const char *name)
  * "checkpasswd" or "environment" method to pass the arguments.
  */
 
-int exec_external(const char *extpath, const char *extmethod,
+static int exec_external(const char *extpath, const char *extmethod,
 		const request_rec *r, const char *dataname, const char *data)
 {
     int pipe_to_auth[2];
@@ -472,36 +493,36 @@ int exec_external(const char *extpath, const char *extmethod,
 	int i= 0;
 
 	/* Put PATH, hostname, ip address, uri, etc into environment */
-	child_env[i++]= ap_pstrcat(r->pool, "PATH=", getenv("PATH"), NULL);
+	child_env[i++]= apr_pstrcat(r->pool, "PATH=", getenv("PATH"), NULL);
 
-	remote_host= ap_get_remote_host(c, r->per_dir_config, REMOTE_HOST);
-	child_env[i++]= ap_pstrcat(r->pool, "AUTHTYPE=", dataname, NULL);
+	remote_host= ap_get_remote_host(c, r->per_dir_config, REMOTE_HOST, NULL);
+	child_env[i++]= apr_pstrcat(r->pool, "AUTHTYPE=", dataname, NULL);
 	if (remote_host != NULL)
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_HOST"=", remote_host, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_HOST"=", remote_host, NULL);
 
 	if (c->remote_ip)
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_IP"=", c->remote_ip, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_IP"=", c->remote_ip, NULL);
 
 	if (r->uri)
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_URI"=", r->uri, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_URI"=", r->uri, NULL);
 
 	if ((host= lookup_header(r,"Host")) != NULL)
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_HTTP_HOST"=", host, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_HTTP_HOST"=", host, NULL);
 
 #ifdef ENV_COOKIE
 	if ((cookie= lookup_header(r,"Cookie")) != NULL)
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_COOKIE"=", cookie, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_COOKIE"=", cookie, NULL);
 #endif
 
 	/* Direct stdout and stderr to log file */
 	ap_error_log2stderr(r->server);
 	dup2(2,1);
 
-	/* Close any open file descriptors and such in the pool. This won't 
+	/* Close any open file descriptors and such in the pool. This won't
 	 * close the pipe because it isn't in the pool.  It will close
 	 * anything that might be on fd 3, which is important to do before
 	 * we try to attach the pipe to that in the checkpassword case.  */
-	ap_cleanup_for_exec();
+	apr_pool_cleanup_for_exec();
 
 	if (usepipe)
 	{
@@ -513,8 +534,8 @@ int exec_external(const char *extpath, const char *extmethod,
 	else
 	{
 	    /* Put user name and password/group into environment */
-	    child_env[i++]= ap_pstrcat(r->pool, ENV_USER"=", c->user, NULL);
-	    child_env[i++]= ap_pstrcat(r->pool, dataname, "=", data, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, ENV_USER"=", r->user, NULL);
+	    child_env[i++]= apr_pstrcat(r->pool, dataname, "=", data, NULL);
 	}
 
 	/* End of environment */
@@ -539,7 +560,7 @@ int exec_external(const char *extpath, const char *extmethod,
 	    close(pipe_to_auth[0]);
 	  
 	    /* Send the user */
-	    write(pipe_to_auth[1], c->user, strlen(c->user));
+	    write(pipe_to_auth[1], r->user, strlen(r->user));
 	    write(pipe_to_auth[1], usecheck ? "\0" : "\n", 1);
 	  
 	    /* Send the password */
@@ -568,19 +589,17 @@ static int extauth_basic_user(request_rec *r)
 {
     extauth_dir_config_rec *dir_config_rec=
 	(extauth_dir_config_rec *)ap_get_module_config(r->per_dir_config,
-	&external_auth_module);
+	&auth_external_module);
 
     extauth_server_config_rec *server_config_rec=
 	(extauth_server_config_rec *)ap_get_module_config(r->server->module_config,
-	&external_auth_module);
+	&auth_external_module);
 
     const char *sent_pw;
     int res, code= 1;
     const char *extname;
     const char *extpath, *extmethod;
 
-    conn_rec *c= r->connection;
-
     /* Get the password, exit if can't get */
     if ((res= ap_get_basic_auth_pw(r, &sent_pw)))
 	    return res;
@@ -593,71 +612,71 @@ static int extauth_basic_user(request_rec *r)
 	return DECLINED;
 
     /* Get the path associated with that external */
-    if (!(extpath= ap_table_get(server_config_rec->auth_extpath, extname)))
+    if (!(extpath= apr_table_get(server_config_rec->auth_extpath, extname)))
     {
 	errno= 0;
-	ap_log_reason(ap_psprintf(r->pool, "invalid AuthExternal keyword (%s)",
-				  extname), r->filename, r);
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+			"invalid AuthExternal keyword (%s)", extname);
 	ap_note_basic_auth_failure(r);
-	return AUTH_REQUIRED;
+	return HTTP_UNAUTHORIZED;
     }
 
     /* Get the Method requested */
-    extmethod= ap_table_get(server_config_rec->auth_extmethod, extname);
+    extmethod= apr_table_get(server_config_rec->auth_extmethod, extname);
 
     if ( extmethod && !strcasecmp(extmethod, "function") )
-#ifdef _HARDCODE_ 
+#ifdef _HARDCODE_
     {
 	char *check_type;		/* Pointer to HARDCODE type check  */
 	char *config_file;		/* Pointer to HARDCODE config file */
 	int standard_auth= 0;
 
 	/* Parse a copy of extpath into type and filename */
-	check_type= ap_pstrdup(r->pool, extpath);
+	check_type= apr_pstrdup(r->pool, extpath);
 	config_file= strchr(check_type, ':');
 	
 	if (config_file == NULL) {        /* No colon! httpd.conf is wrong */
 	    errno= 0;
-	    ap_log_reason(ap_psprintf(r->pool,
-	    			      "Problem parsing config file: "
-	    			      "(%s) directive. You must have a ':' "
-	    			      "seperator!\n", check_type),
-			  r->filename, r);
-	    return SERVER_ERROR;
+	    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+	    	"Problem parsing config file: "
+	    	"(%s) directive. You must have a ':' "
+	    	"seperator!\n", check_type);
+	    	
+	    return HTTP_INTERNAL_SERVER_ERROR;
 	}
 	*config_file= '\0';		    /* Mark end of type */
 	config_file++;                      /* Start of filename */
 
-	/* This is where you make your function call.       */
-	/* Here is an example of what one looks like.       */
-	/*                                                  */
-	/* if (strcmp(check_type,"RADIUS")==0)              */
-	/*   code= radcheck(c->user,sent_pw,config_file);  */
-	/*                                                  */
-	/* Replace 'RADIUS' with whatever you are using as  */
-	/* the <type> in:                                   */
-	/* AddExternalAuth <keyword> <type>:<config file>   */
-	/*                                                  */
-	/* Replace radcheck with whatever the name of your  */
-	/* function is.                                     */
-	/* Note: If you dont use a config_file you must at  */
-	/* least include the ':'..for example..at a minimum */
+	/* This is where you make your function call.         */
+	/* Here is an example of what one looks like.         */
+	/*                                                    */
+	/* if (strcmp(check_type,"RADIUS")==0)                */
+	/*   code= radcheck(r->user,sent_pw,config_file);     */
+	/*                                                    */
+	/* Replace 'RADIUS' with whatever you are using as    */
+	/* the <type> in:                                     */
+	/* AddExternalAuth <keyword> <type>:<config file>     */
+	/*                                                    */
+	/* Replace radcheck with whatever the name of your    */
+	/* function is.                                       */
+	/* Note: If you dont use a config_file you must at    */
+	/* least include the ':'..for example..at a minimum   */
 	/* you must have 'RADIUS:' in the httpd.conf for your */
-	/* function to be called.                           */
+	/* function to be called.                             */
 
 	else
 	{
 	    errno= 0;
-	    ap_log_reason(ap_psprintf(r->pool,"Problem parsing config file: "
-	    		              "(%s) directive (%s) config_file\n",
-	    		              check_type, config_file),
-			  r->filename, r);
-	    return SERVER_ERROR;
+	    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+		"Problem parsing config file: "
+		"(%s) directive (%s) config_file\n",
+		check_type, config_file);
+	    return HTTP_INTERNAL_SERVER_ERROR;
 	}
     }
 #else
-	  /* If _HARDCODE_ is not defined than no mater what the user says */
-	  /* about wanting to use a function we fail!!                     */
+	  /* If _HARDCODE_ is not defined then no matter what the user says */
+	  /* about wanting to use a function we fail!!                      */
 	  code= -4;
 #endif
     else
@@ -670,11 +689,11 @@ static int extauth_basic_user(request_rec *r)
     if (!dir_config_rec->external_authoritative)
     	return DECLINED;
 
-    ap_log_reason(ap_psprintf(r->pool,"AuthExtern %s [%s]: Failed (%d) for "
-    			      "user %s", extname, extpath, code, c->user),
-		  r->filename, r);
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+    	"AuthExtern %s [%s]: Failed (%d) for "
+    	"user %s", extname, extpath, code, r->user);
     ap_note_basic_auth_failure(r);
-    return AUTH_REQUIRED;
+    return HTTP_UNAUTHORIZED;
 }
 
 
@@ -682,20 +701,18 @@ static int extauth_check_auth(request_rec *r)
 {
     extauth_dir_config_rec *dir_config_rec=
 	(extauth_dir_config_rec *)ap_get_module_config(r->per_dir_config,
-	&external_auth_module);
+	&auth_external_module);
 
     extauth_server_config_rec *server_config_rec=
 	(extauth_server_config_rec *)ap_get_module_config(r->server->module_config,
-	&external_auth_module);
-
-    conn_rec *c= r->connection;
+	&auth_external_module);
 
     int m= r->method_number;
     int code;
     char *extname;
     const char *extpath, *extmethod;
 
-    const array_header *reqs_arr= ap_requires(r);
+    const apr_array_header_t *reqs_arr= ap_requires(r);
     require_line *reqs= reqs_arr ? (require_line *)reqs_arr->elts : NULL;
 
     int x;
@@ -712,7 +729,7 @@ static int extauth_check_auth(request_rec *r)
     if (!reqs_arr)
     	return DECLINED;
 
-    for(x=0; x < reqs_arr->nelts; x++) 
+    for(x=0; x < reqs_arr->nelts; x++)
     {
 	if (!(reqs[x].method_mask & (1 << m))) continue;
 
@@ -722,31 +739,30 @@ static int extauth_check_auth(request_rec *r)
 	if(!strcmp(w,"valid-user"))
 	    return OK;
 
-	if(!strcmp(w,"user")) 
+	if(!strcmp(w,"user"))
 	{
-	    while(t[0]) 
+	    while(t[0])
 	    {
 		w= ap_getword_conf(r->pool, &t);
-		if(!strcmp(c->user,w))
+		if(!strcmp(r->user,w))
 		    return OK;
 	    }
 	}
-	else if( !strcmp(w,"group") ) 
+	else if( !strcmp(w,"group") )
 	{
 	    if (t[0])
 	    {
 		/* Get the path and method associated with that external */
-		if (!(extpath= ap_table_get(
+		if (!(extpath= apr_table_get(
 			server_config_rec->group_extpath, extname)) ||
-		    !(extmethod= ap_table_get(
+		    !(extmethod= apr_table_get(
 		    	server_config_rec->group_extmethod, extname)))
 		{
 		    errno= 0;
-		    ap_log_reason(ap_psprintf(r->pool,
-			"invalid GroupExternal keyword (%s)", extname),
-			r->filename, r);
+		    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+		    	"invalid GroupExternal keyword (%s)", extname);
 		    ap_note_basic_auth_failure(r);
-		    return AUTH_REQUIRED;
+		    return HTTP_UNAUTHORIZED;
 		}
 
 		if (dir_config_rec->external_groupsatonce)
@@ -773,7 +789,7 @@ static int extauth_check_auth(request_rec *r)
 	     * valid even if we don't grok it.  However if we are, we can
 	     * warn the user that they did something wrong.
 	     */
-	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
+	    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
 	    	"access to %s failed, reason: unknown require directive:"
 	    	"\"%s\"", r->uri, reqs[x].requirement);
 	}
@@ -782,28 +798,28 @@ static int extauth_check_auth(request_rec *r)
     if (!dir_config_rec->external_authoritative)
 	return DECLINED;
 
-    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
     	"access to %s failed, reason: user %s not allowed access",
-    	r->uri, c->user);
+    	r->uri, r->user);
 
     ap_note_basic_auth_failure(r);
-    return AUTH_REQUIRED;
+    return HTTP_UNAUTHORIZED;
 }
 
-module external_auth_module= {
-    STANDARD_MODULE_STUFF,
-    NULL,			 /* initializer */
-    create_extauth_dir_config,	 /* dir config creater */
-    NULL,			 /* dir merger --- default is to override */
-    create_extauth_server_config,/* server config */
-    NULL,			 /* merge server config */
-    extauth_cmds,		 /* command table */
-    NULL,			 /* handlers */
-    NULL,			 /* filename translation */
-    extauth_basic_user,		 /* check_user_id */
-    extauth_check_auth,		 /* check auth */
-    NULL,			 /* check access */
-    NULL,			 /* type_checker */
-    NULL,			 /* fixups */
-    NULL			 /* logger */
+
+static void register_hooks(apr_pool_t *p)
+{
+    ap_hook_check_user_id(extauth_basic_user, NULL, NULL, APR_HOOK_MIDDLE);
+    ap_hook_auth_checker(extauth_check_auth, NULL, NULL, APR_HOOK_MIDDLE);
+}
+    
+
+module AP_MODULE_DECLARE_DATA auth_external_module = {
+    STANDARD20_MODULE_STUFF,
+    create_extauth_dir_config,	  /* create per-dir config */
+    NULL,			  /* merge per-dir config - dflt is override */
+    create_extauth_server_config, /* create per-server config */
+    NULL,			  /* merge per-server config */
+    extauth_cmds,		  /* command apr_table_t */
+    register_hooks		  /* register hooks */
 };
